@@ -1,45 +1,6 @@
-# ***** BEGIN LICENSE BLOCK *****
-# Version: MPL 1.1/GPL 2.0/LGPL 2.1
-#
-# The contents of this file are subject to the Mozilla Public License Version
-# 1.1 (the "License"); you may not use this file except in compliance with
-# the License. You may obtain a copy of the License at
-# http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS IS" basis,
-# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
-# for the specific language governing rights and limitations under the
-# License.
-#
-# The Original Code is Mozilla Addressbook.
-#
-# The Initial Developer of the Original Code is
-# Netscape Communications Corp.
-# Portions created by the Initial Developer are Copyright (C) 1999-2001
-# the Initial Developer. All Rights Reserved.
-#
-# Original Author:
-#   Paul Hangas <hangas@netscape.com>
-#
-# Contributor(s):
-#   Seth Spitzer <sspitzer@netscape.com>
-#   Mark Banner <mark@standard8.demon.co.uk>
-#   Simon Wilkinson <simon@sxw.org.uk>
-#   Mike Conley <mconley@mozilla.com>
-#
-# Alternatively, the contents of this file may be used under the terms of
-# either the GNU General Public License Version 2 or later (the "GPL"), or
-# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
-# in which case the provisions of the GPL or the LGPL are applicable instead
-# of those above. If you wish to allow use of your version of this file only
-# under the terms of either the GPL or the LGPL, and not to allow others to
-# use your version of this file under the terms of the MPL, indicate your
-# decision by deleting the provisions above and replace them with the notice
-# and other provisions required by the GPL or the LGPL. If you do not delete
-# the provisions above, a recipient may use your version of this file under
-# the terms of any one of the MPL, the GPL or the LGPL.
-#
-# ***** END LICENSE BLOCK *****
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource:///modules/mailServices.js");
@@ -56,8 +17,12 @@ const kDefaultDescending = "descending";
 const kLdapUrlPrefix = "moz-abldapdirectory://";
 const kPersonalAddressbookURI = "moz-abmdbdirectory://abook.mab";
 const kCollectedAddressbookURI = "moz-abmdbdirectory://history.mab";
-// The default image for contacts
-var defaultPhotoURI = "chrome://messenger/skin/addressbook/icons/contact-generic.png";
+// The default, generic contact image is displayed via CSS when the photoURI is
+// blank.
+let defaultPhotoURI = "";
+
+const PERMS_FILE = parseInt("0644", 8);
+const PERMS_DIRECTORY = parseInt("0755", 8);
 
 // Controller object for Dir Pane
 var DirPaneController =
@@ -109,7 +74,7 @@ var DirPaneController =
           // If the selected directory is an ldap directory
           // and if the prefs for this directory are locked
           // disable the delete button.
-          if (selectedDir.lastIndexOf(kLdapUrlPrefix, 0) == 0)
+          if (selectedDir.startsWith(kLdapUrlPrefix))
           {
             var disable = false;
             try {
@@ -561,11 +526,11 @@ function InitViewSortByMenu()
     }
 
     // this approach is necessary to support generic columns that get overlayed.
-    var elements = document.getElementsByAttribute("name","sortas");
-    for (var i=0; i<elements.length; i++) {
-        var cmd = elements[i].getAttribute("id");
-        var columnForCmd = cmd.split("cmd_SortBy")[1];
-        setSortByMenuItemCheckState(cmd, (sortColumn == columnForCmd));
+    let elements = document.querySelectorAll('[name="sortas"]');
+    for (let i = 0; i < elements.length; i++) {
+      let cmd = elements[i].id;
+      let columnForCmd = cmd.substr(10); // everything right of cmd_SortBy
+      setSortByMenuItemCheckState(cmd, (sortColumn == columnForCmd));
     }
 
     setSortByMenuItemCheckState("sortAscending", (sortDirection == kDefaultAscending));
@@ -647,251 +612,6 @@ function QuickSearchFocus()
   }
 }
 
-var gQuickSearchFocusEl = null;
-var gIsOffline;
-var gSessionAdded;
-var gCurrentAutocompleteDirectory;
-var gAutocompleteSession;
-var gSetupLdapAutocomplete;
-var gLDAPSession;
-
-function setupLdapAutocompleteSession()
-{
-    var autocompleteLdap = false;
-    var autocompleteDirectory = null;
-    var prevAutocompleteDirectory = gCurrentAutocompleteDirectory;
-    var i;
-
-    autocompleteLdap = Services.prefs.getBoolPref("ldap_2.autoComplete.useDirectory");
-    if (autocompleteLdap)
-        autocompleteDirectory = Services.prefs.getCharPref(
-            "ldap_2.autoComplete.directoryServer");
-
-
-    // use a temporary to do the setup so that we don't overwrite the
-    // global, then have some problem and throw an exception, and leave the
-    // global with a partially setup session.  we'll assign the temp
-    // into the global after we're done setting up the session
-    //
-    var LDAPSession;
-    if (gLDAPSession) {
-        LDAPSession = gLDAPSession;
-    } else {
-        LDAPSession = Components.classes[
-            "@mozilla.org/autocompleteSession;1?type=ldap"].createInstance()
-            .QueryInterface(Components.interfaces.nsILDAPAutoCompleteSession);
-    }
-
-    if (autocompleteDirectory && !gIsOffline) {
-        // the compose window code adds an observer on the directory server
-        // prefs, but I don't think we need this here.
-        gCurrentAutocompleteDirectory = autocompleteDirectory;
-
-        // fill in the session params if there is a session
-        //
-        if (LDAPSession) {
-            let url =
-              Services.prefs.getComplexValue(autocompleteDirectory + ".uri",
-                Components.interfaces.nsISupportsString).data;
-
-            LDAPSession.serverURL = Services.io.newURI(url, null, null)
-              .QueryInterface(Components.interfaces.nsILDAPURL);
-
-            // get the login to authenticate as, if there is one
-            //
-            try {
-                LDAPSession.login = Services.prefs.getComplexValue(
-                    autocompleteDirectory + ".auth.dn",
-                    Components.interfaces.nsISupportsString).data;
-            } catch (ex) {
-                // if we don't have this pref, no big deal
-            }
-
-            try {
-                LDAPSession.saslMechanism = Services.prefs.getComplexValue(
-                    autocompleteDirectory + ".auth.saslmech",
-                    Components.interfaces.nsISupportsString).data;
-            } catch (ex) {
-                // if we don't have a mechanism, we'll just use simple binds
-            }
-
-            // don't search on non-CJK strings shorter than this
-            //
-            try {
-                LDAPSession.minStringLength = Services.prefs.getIntPref(
-                    autocompleteDirectory + ".autoComplete.minStringLength");
-            } catch (ex) {
-                // if this pref isn't there, no big deal.  just let
-                // nsLDAPAutoCompleteSession use its default.
-            }
-
-            // don't search on CJK strings shorter than this
-            //
-            try {
-                LDAPSession.cjkMinStringLength = Services.prefs.getIntPref(
-                  autocompleteDirectory + ".autoComplete.cjkMinStringLength");
-            } catch (ex) {
-                // if this pref isn't there, no big deal.  just let
-                // nsLDAPAutoCompleteSession use its default.
-            }
-
-            // we don't try/catch here, because if this fails, we're outta luck
-            //
-            var ldapFormatter = Components.classes[
-                "@mozilla.org/ldap-autocomplete-formatter;1?type=addrbook"]
-                .createInstance().QueryInterface(
-                    Components.interfaces.nsIAbLDAPAutoCompFormatter);
-
-            // override autocomplete name format?
-            //
-            try {
-                ldapFormatter.nameFormat =
-                    Services.prefs.getComplexValue(autocompleteDirectory +
-                        ".autoComplete.nameFormat",
-                        Components.interfaces.nsISupportsString).data;
-            } catch (ex) {
-                // if this pref isn't there, no big deal.  just let
-                // nsAbLDAPAutoCompFormatter use its default.
-            }
-
-            // override autocomplete mail address format?
-            //
-            try {
-                ldapFormatter.addressFormat =
-                    Services.prefs.getComplexValue(autocompleteDirectory +
-                        ".autoComplete.addressFormat",
-                        Components.interfaces.nsISupportsString).data;
-            } catch (ex) {
-                // if this pref isn't there, no big deal.  just let
-                // nsAbLDAPAutoCompFormatter use its default.
-            }
-
-            try {
-                // figure out what goes in the comment column, if anything
-                //
-                // 0 = none
-                // 1 = name of addressbook this card came from
-                // 2 = other per-addressbook format
-                //
-                var showComments = 0;
-                showComments = Services.prefs.getIntPref(
-                    "mail.autoComplete.commentColumn");
-
-                switch (showComments) {
-
-                case 1:
-                    // use the name of this directory
-                    //
-                    ldapFormatter.commentFormat = Services.prefs.getComplexValue(
-                        autocompleteDirectory + ".description",
-                        Components.interfaces.nsISupportsString).data;
-                    break;
-
-                case 2:
-                    // override ldap-specific autocomplete entry?
-                    //
-                    try {
-                        ldapFormatter.commentFormat =
-                            Services.prefs.getComplexValue(autocompleteDirectory +
-                                ".autoComplete.commentFormat",
-                                Components.interfaces.nsISupportsString).data;
-                    } catch (innerException) {
-                        // if nothing has been specified, use the ldap
-                        // organization field
-                        ldapFormatter.commentFormat = "[o]";
-                    }
-                    break;
-
-                case 0:
-                default:
-                    // do nothing
-                }
-            } catch (ex) {
-                // if something went wrong while setting up comments, try and
-                // proceed anyway
-            }
-
-            // set the session's formatter, which also happens to
-            // force a call to the formatter's getAttributes() method
-            // -- which is why this needs to happen after we've set the
-            // various formats
-            //
-            LDAPSession.formatter = ldapFormatter;
-
-            // override autocomplete entry formatting?
-            //
-            try {
-                LDAPSession.outputFormat =
-                    Services.prefs.getComplexValue(autocompleteDirectory +
-                        ".autoComplete.outputFormat",
-                        Components.interfaces.nsISupportsString).data;
-
-            } catch (ex) {
-                // if this pref isn't there, no big deal.  just let
-                // nsLDAPAutoCompleteSession use its default.
-            }
-
-            // override default search filter template?
-            //
-            try {
-                LDAPSession.filterTemplate = Services.prefs.getComplexValue(
-                    autocompleteDirectory + ".autoComplete.filterTemplate",
-                    Components.interfaces.nsISupportsString).data;
-
-            } catch (ex) {
-                // if this pref isn't there, no big deal.  just let
-                // nsLDAPAutoCompleteSession use its default
-            }
-
-            // override default maxHits (currently 100)
-            //
-            try {
-                // XXXdmose should really use .autocomplete.maxHits,
-                // but there's no UI for that yet
-                //
-                LDAPSession.maxHits =
-                    Services.prefs.getIntPref(autocompleteDirectory + ".maxHits");
-            } catch (ex) {
-                // if this pref isn't there, or is out of range, no big deal.
-                // just let nsLDAPAutoCompleteSession use its default.
-            }
-
-            if (!gSessionAdded) {
-                // if we make it here, we know that session initialization has
-                // succeeded; add the session for all recipients, and
-                // remember that we've done so
-                var autoCompleteWidget;
-                for (i=1; i <= awGetMaxRecipients(); i++)
-                {
-                    autoCompleteWidget = document.getElementById("addressCol1#" + i);
-                    if (autoCompleteWidget)
-                    {
-                      autoCompleteWidget.addSession(LDAPSession);
-                      // ldap searches don't insert a default entry with the default domain appended to it
-                      // so reduce the minimum results for a popup to 2 in this case.
-                      autoCompleteWidget.minResultsForPopup = 2;
-
-                    }
-                 }
-                gSessionAdded = true;
-            }
-        }
-    } else {
-      if (gCurrentAutocompleteDirectory) {
-        gCurrentAutocompleteDirectory = null;
-      }
-      if (gLDAPSession && gSessionAdded) {
-        for (i=1; i <= awGetMaxRecipients(); i++)
-          document.getElementById("addressCol1#" + i).
-              removeSession(gLDAPSession);
-        gSessionAdded = false;
-      }
-    }
-
-    gLDAPSession = LDAPSession;
-    gSetupLdapAutocomplete = true;
-}
-
 /**
  * Returns an nsIFile of the directory in which contact photos are stored.
  * This will create the directory if it does not yet exist.
@@ -901,7 +621,7 @@ function getPhotosDir() {
   // Get the Photos directory
   file.append("Photos");
   if (!file.exists() || !file.isDirectory())
-    file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
+    file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, PERMS_DIRECTORY);
   return file;
 }
 
@@ -945,7 +665,7 @@ function saveStreamToFile(aIStream, aFile) {
                           .createInstance(Components.interfaces.nsIFileOutputStream);
   var buffer  = Components.classes["@mozilla.org/network/buffered-output-stream;1"]
                           .createInstance(Components.interfaces.nsIBufferedOutputStream);
-  fstream.init(aFile, 0x04 | 0x08 | 0x20, 0600, 0); // write, create, truncate
+  fstream.init(aFile, 0x04 | 0x08 | 0x20, PERMS_FILE, 0); // write, create, truncate
   buffer.init(fstream, 8192);
 
   buffer.writeFrom(aIStream, aIStream.available());
