@@ -10,9 +10,7 @@ MOZ_THUNDERBIRD=1
 MOZ_CHROME_FILE_FORMAT=omni
 MOZ_NO_ACTIVEX_SUPPORT=1
 MOZ_ACTIVEX_SCRIPTING_SUPPORT=
-if [ "$COMM_BUILD" ]; then
-  MOZ_LDAP_XPCOM=1
-fi
+MOZ_LDAP_XPCOM=1
 MOZ_COMPOSER=1
 if test "$OS_ARCH" = "WINNT"; then
   if ! test "$HAVE_64BIT_OS"; then
@@ -21,16 +19,19 @@ if test "$OS_ARCH" = "WINNT"; then
   fi
 fi
 
-# Disable WebRTC as we don't use it currently and to work around bug 837618
-# for Mac
-MOZ_WEBRTC=
+# Disable Accessibility on Mac for now as unit tests fail (bug 862238).
+if test "$OS_ARCH" = "Darwin"; then
+  ACCESSIBILITY=
+fi
 
 MOZ_SAFE_BROWSING=1
+MOZ_MEDIA_NAVIGATOR=1
 MOZ_MORK=1
-if test -z "$MOZ_INCOMPLETE_EXTERNAL_LINKAGE"; then
-MOZ_APP_COMPONENT_LIBS="xpautocomplete mailcomps $MAIL_COMPONENT $LDAP_COMPONENT $MORK_COMPONENT"
-MOZ_APP_COMPONENT_MODULES="MODULE(xpAutoComplete) MODULE(nsMailCompsModule) $MAIL_MODULE $LDAP_MODULE $MORK_MODULE"
-MOZ_APP_EXTRA_LIBS="$LDAP_LIBS"
+MAIL_COMPONENT="mail msgsmime import"
+MAIL_MODULE="MODULE(nsMailModule) MODULE(nsMsgSMIMEModule) MODULE(nsImportServiceModule)"
+if test -n "$MOZ_MAPI_SUPPORT"; then
+  MAIL_COMPONENT="$MAIL_COMPONENT msgMapi"
+  MAIL_MODULE="$MAIL_MODULE MODULE(msgMapiModule)"
 fi
 
 MOZ_APP_VERSION_TXT=${_topsrcdir}/$MOZ_BUILD_APP/config/version.txt
