@@ -40,6 +40,7 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsIRssIncomingServer.h"
 #include "nsIMsgFolder.h"
+#include "nsIMsgProtocolInfo.h"
 #include "nsIMsgMessageService.h"
 #include "nsIMsgAccountManager.h"
 #include "nsIOutputStream.h"
@@ -523,7 +524,7 @@ nsresult FormatFileSize(uint64_t size, bool useKB, nsAString &formattedSize)
     formattedSize, sizeAbbr.get(),
     (unitIndex != 0) && (unitSize < 99.95 && unitSize != 0) ? 1 : 0, unitSize);
 
-  PRInt32 separatorPos = formattedSize.FindChar('.');
+  int32_t separatorPos = formattedSize.FindChar('.');
   if (separatorPos != kNotFound) {
     // The ssprintf returned a decimal number using a dot (.) as the decimal
     // separator. Now we try to localize the separator.
@@ -619,7 +620,6 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *aFolderURI,
 bool NS_MsgStripRE(const char **stringP, uint32_t *lengthP, char **modifiedSubject)
 {
   const char *s, *s_end;
-  const char *last;
   uint32_t L;
   bool result = false;
   NS_ASSERTION(stringP, "bad null param");
@@ -657,7 +657,6 @@ bool NS_MsgStripRE(const char **stringP, uint32_t *lengthP, char **modifiedSubje
   L = lengthP ? *lengthP : strlen(s);
 
   s_end = s + L;
-  last = s;
 
  AGAIN:
 
@@ -765,28 +764,20 @@ char * NS_MsgSACopy (char **destination, const char *source)
   return *destination;
 }
 
-/*  Again like strdup but it concatinates and free's and uses Realloc
+/*  Again like strdup but it concatenates and free's and uses Realloc.
 */
 char * NS_MsgSACat (char **destination, const char *source)
 {
   if (source && *source)
-    if (*destination)
-    {
-      int length = PL_strlen (*destination);
-      *destination = (char *) PR_Realloc (*destination, length + PL_strlen(source) + 1);
-      if (*destination == nullptr)
-        return(nullptr);
+  {
+    int destLength = *destination ? PL_strlen(*destination) : 0;
+    char* newDestination = (char*) PR_Realloc(*destination, destLength + PL_strlen(source) + 1);
+    if (newDestination == nullptr)
+      return nullptr;
 
-      PL_strcpy (*destination + length, source);
-    }
-    else
-    {
-      *destination = (char *) PR_Malloc (PL_strlen(source) + 1);
-      if (*destination == nullptr)
-        return(nullptr);
-
-      PL_strcpy (*destination, source);
-    }
+    *destination = newDestination;
+    PL_strcpy(*destination + destLength, source);
+  }
   return *destination;
 }
 
@@ -959,16 +950,19 @@ GetOrCreateFolder(const nsACString &aURI, nsIUrlListener *aListener)
     // for imap folders, path needs to have .msf appended to the name
     msgFolder->GetFilePath(getter_AddRefs(folderPath));
 
-    nsCString type;
-    rv = server->GetType(type);
-    NS_ENSURE_SUCCESS(rv,rv);
+    nsCOMPtr<nsIMsgProtocolInfo> protocolInfo;
+    rv = server->GetProtocolInfo(getter_AddRefs(protocolInfo));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    bool isAsyncFolder;
+    rv = protocolInfo->GetFoldersCreatedAsync(&isAsyncFolder);
+    NS_ENSURE_SUCCESS(rv, rv);
 
-    bool isImapFolder = type.Equals("imap");
     // if we can't get the path from the folder, then try to create the storage.
     // for imap, it doesn't matter if the .msf file exists - it still might not
     // exist on the server, so we should try to create it
     bool exists = false;
-    if (!isImapFolder && folderPath)
+    if (!isAsyncFolder && folderPath)
       folderPath->Exists(&exists);
     if (!exists)
     {
@@ -996,7 +990,7 @@ GetOrCreateFolder(const nsACString &aURI, nsIUrlListener *aListener)
       // we should look into making it so no matter what the folder type
       // we always call the listener
       // this code should move into local folder's version of CreateStorageIfMissing()
-      if (!isImapFolder && aListener) {
+      if (!isAsyncFolder && aListener) {
         rv = aListener->OnStartRunningUrl(nullptr);
         NS_ENSURE_SUCCESS(rv,rv);
 
@@ -1900,14 +1894,14 @@ NS_MSG_BASE void MsgReplaceChar(nsCString& str, const char needle, const char re
   }
 }
 
-NS_MSG_BASE nsIAtom* MsgNewAtom(const char* aString)
+NS_MSG_BASE already_AddRefed<nsIAtom> MsgNewAtom(const char* aString)
 {
   nsCOMPtr<nsIAtomService> atomService(do_GetService("@mozilla.org/atom-service;1"));
-  nsIAtom* atom = nullptr;
+  nsCOMPtr<nsIAtom> atom;
 
   if (atomService)
-    atomService->GetAtomUTF8(aString, &atom);
-  return atom;
+    atomService->GetAtomUTF8(aString, getter_AddRefs(atom));
+  return atom.forget();
 }
 
 NS_MSG_BASE nsIAtom* MsgNewPermanentAtom(const char* aString)
@@ -1955,7 +1949,7 @@ NS_MSG_BASE void MsgReplaceSubstring(nsACString &str, const char *what, const ch
 class MsgInterfaceRequestorAgg : public nsIInterfaceRequestor
 {
 public:
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIINTERFACEREQUESTOR
 
   MsgInterfaceRequestorAgg(nsIInterfaceRequestor *aFirst,
@@ -1967,7 +1961,7 @@ public:
 };
 
 // XXX This needs to support threadsafe refcounting until we fix bug 243591.
-NS_IMPL_THREADSAFE_ISUPPORTS1(MsgInterfaceRequestorAgg, nsIInterfaceRequestor)
+NS_IMPL_ISUPPORTS1(MsgInterfaceRequestorAgg, nsIInterfaceRequestor)
 
 NS_IMETHODIMP
 MsgInterfaceRequestorAgg::GetInterface(const nsIID &aIID, void **aResult)
@@ -2011,6 +2005,8 @@ nsresult NS_FASTCALL MsgQueryElementAt::operator()( const nsIID& aIID, void** aR
 NS_MSG_BASE nsresult MsgGetHeadersFromKeys(nsIMsgDatabase *aDB, const nsTArray<nsMsgKey> &aMsgKeys,
                                            nsIMutableArray *aHeaders)
 {
+  NS_ENSURE_ARG_POINTER(aDB);
+
   uint32_t count = aMsgKeys.Length();
   nsresult rv = NS_OK;
 
