@@ -1,43 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Lorenzo Colitti <lorenzo@colitti.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #ifndef nsImapMailFolder_h__
 #define nsImapMailFolder_h__
 
+#include "mozilla/Attributes.h"
 #include "nsImapCore.h"
 #include "nsMsgDBFolder.h"
 #include "nsIImapMailFolderSink.h"
@@ -81,7 +49,7 @@ class nsImapMailCopyState: public nsISupports
 public:
     NS_DECLARE_STATIC_IID_ACCESSOR(NS_IMAPMAILCOPYSTATE_IID)
 
-    NS_DECL_ISUPPORTS
+    NS_DECL_THREADSAFE_ISUPPORTS
 
     nsImapMailCopyState();
     virtual ~nsImapMailCopyState();
@@ -100,18 +68,18 @@ public:
     bool m_isMove;             // is a move
     bool m_selectedState;      // needs to be in selected state; append msg
     bool m_isCrossServerOp; // are we copying between imap servers?
-    PRUint32 m_curIndex; // message index to the message array which we are
+    uint32_t m_curIndex; // message index to the message array which we are
                          // copying
-    PRUint32 m_totalCount;// total count of messages we have to do
-    PRUint32 m_unreadCount; // num unread messages we're moving
+    uint32_t m_totalCount;// total count of messages we have to do
+    uint32_t m_unreadCount; // num unread messages we're moving
     bool m_streamCopy;
     char *m_dataBuffer; // temporary buffer for this copy operation
     nsCOMPtr<nsIOutputStream> m_msgFileStream;         // temporary file (processed mail)
-    PRUint32 m_dataBufferSize;
-    PRUint32 m_leftOver;
+    uint32_t m_dataBufferSize;
+    uint32_t m_leftOver;
     bool m_allowUndo;
     bool m_eatLF;
-    PRUint32 m_newMsgFlags; // only used if there's no m_message
+    uint32_t m_newMsgFlags; // only used if there's no m_message
     nsCString m_newMsgKeywords; // ditto 
     // If the server supports UIDPLUS, this is the UID for the append,
     // if we're doing an append.
@@ -199,7 +167,7 @@ protected:
 protected:
   nsDataHashtable <nsCStringHashKey, nsCString> m_rightsHash; // Hash table, mapping username strings to rights strings.
   nsImapMailFolder *m_folder;
-  PRInt32        m_aclCount;
+  int32_t        m_aclCount;
 
 };
 
@@ -224,7 +192,7 @@ class nsImapMailFolder :  public nsMsgDBFolder,
                           public nsICopyMessageListener,
                           public nsIMsgFilterHitNotify
 {
- static const PRUint32 PLAYBACK_TIMER_INTERVAL_IN_MS = 500; 
+ static const uint32_t PLAYBACK_TIMER_INTERVAL_IN_MS = 500; 
 public:
   nsImapMailFolder();
   virtual ~nsImapMailFolder();
@@ -232,94 +200,98 @@ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIMsgFolder methods:
-  NS_IMETHOD GetSubFolders(nsISimpleEnumerator **aResult);
+  NS_IMETHOD GetSubFolders(nsISimpleEnumerator **aResult) MOZ_OVERRIDE;
 
-  NS_IMETHOD GetMessages(nsISimpleEnumerator* *result);
-  NS_IMETHOD UpdateFolder(nsIMsgWindow *aWindow);
+  NS_IMETHOD GetMessages(nsISimpleEnumerator* *result) MOZ_OVERRIDE;
+  NS_IMETHOD UpdateFolder(nsIMsgWindow *aWindow) MOZ_OVERRIDE;
 
-  NS_IMETHOD CreateSubfolder(const nsAString& folderName,nsIMsgWindow *msgWindow );
-  NS_IMETHOD AddSubfolder(const nsAString& aName, nsIMsgFolder** aChild);
+  NS_IMETHOD CreateSubfolder(const nsAString& folderName,nsIMsgWindow *msgWindow ) MOZ_OVERRIDE;
+  NS_IMETHOD AddSubfolder(const nsAString& aName, nsIMsgFolder** aChild) MOZ_OVERRIDE;
   NS_IMETHODIMP CreateStorageIfMissing(nsIUrlListener* urlListener);
 
-  NS_IMETHOD Compact(nsIUrlListener *aListener, nsIMsgWindow *aMsgWindow);
+  NS_IMETHOD Compact(nsIUrlListener *aListener, nsIMsgWindow *aMsgWindow) MOZ_OVERRIDE;
   NS_IMETHOD CompactAll(nsIUrlListener *aListener, nsIMsgWindow *aMsgWindow,
-                        bool aCompactOfflineAlso);
-  NS_IMETHOD EmptyTrash(nsIMsgWindow *msgWindow, nsIUrlListener *aListener);
+                        bool aCompactOfflineAlso) MOZ_OVERRIDE;
+  NS_IMETHOD EmptyTrash(nsIMsgWindow *msgWindow, nsIUrlListener *aListener) MOZ_OVERRIDE;
   NS_IMETHOD CopyDataToOutputStreamForAppend(nsIInputStream *aIStream,
-                     PRInt32 aLength, nsIOutputStream *outputStream);
-  NS_IMETHOD CopyDataDone();
-  NS_IMETHOD Delete ();
-  NS_IMETHOD Rename (const nsAString& newName, nsIMsgWindow *msgWindow);
-  NS_IMETHOD RenameSubFolders(nsIMsgWindow *msgWindow, nsIMsgFolder *oldFolder);
-  NS_IMETHOD GetNoSelect(bool *aResult);
+                     int32_t aLength, nsIOutputStream *outputStream) MOZ_OVERRIDE;
+  NS_IMETHOD CopyDataDone() MOZ_OVERRIDE;
+  NS_IMETHOD Delete () MOZ_OVERRIDE;
+  NS_IMETHOD Rename (const nsAString& newName, nsIMsgWindow *msgWindow) MOZ_OVERRIDE;
+  NS_IMETHOD RenameSubFolders(nsIMsgWindow *msgWindow, nsIMsgFolder *oldFolder) MOZ_OVERRIDE;
+  NS_IMETHOD GetNoSelect(bool *aResult) MOZ_OVERRIDE;
 
-  NS_IMETHOD GetPrettyName(nsAString& prettyName); // Override of the base, for top-level mail folder
+  NS_IMETHOD GetPrettyName(nsAString& prettyName) MOZ_OVERRIDE; // Override of the base, for top-level mail folder
 
-  NS_IMETHOD GetFolderURL(nsACString& url);
+  NS_IMETHOD GetFolderURL(nsACString& url) MOZ_OVERRIDE;
 
-  NS_IMETHOD UpdateSummaryTotals(bool force) ;
+  NS_IMETHOD UpdateSummaryTotals(bool force) MOZ_OVERRIDE;
 
-  NS_IMETHOD GetDeletable (bool *deletable);
-  NS_IMETHOD GetRequiresCleanup(bool *requiresCleanup);
+  NS_IMETHOD GetDeletable (bool *deletable) MOZ_OVERRIDE;
 
-  NS_IMETHOD GetSizeOnDisk(PRUint32 * size);
+  NS_IMETHOD GetSizeOnDisk(uint32_t * size) MOZ_OVERRIDE;
 
-  NS_IMETHOD GetCanCreateSubfolders(bool *aResult);
-  NS_IMETHOD GetCanSubscribe(bool *aResult);
+  NS_IMETHOD GetCanCreateSubfolders(bool *aResult) MOZ_OVERRIDE;
+  NS_IMETHOD GetCanSubscribe(bool *aResult) MOZ_OVERRIDE;
 
-  NS_IMETHOD ApplyRetentionSettings();
+  NS_IMETHOD ApplyRetentionSettings() MOZ_OVERRIDE;
 
-  NS_IMETHOD AddMessageDispositionState(nsIMsgDBHdr *aMessage, nsMsgDispositionState aDispositionFlag);
-  NS_IMETHOD MarkMessagesRead(nsIArray *messages, bool markRead);
-  NS_IMETHOD MarkAllMessagesRead(nsIMsgWindow *aMsgWindow);
-  NS_IMETHOD MarkMessagesFlagged(nsIArray *messages, bool markFlagged);
-  NS_IMETHOD MarkThreadRead(nsIMsgThread *thread);
-  NS_IMETHOD SetLabelForMessages(nsIArray *aMessages, nsMsgLabelValue aLabel);
-  NS_IMETHOD SetJunkScoreForMessages(nsIArray *aMessages, const nsACString& aJunkScore);
-  NS_IMETHOD DeleteSubFolders(nsIArray *folders, nsIMsgWindow *msgWindow);
-  NS_IMETHOD ReadFromFolderCacheElem(nsIMsgFolderCacheElement *element);
-  NS_IMETHOD WriteToFolderCacheElem(nsIMsgFolderCacheElement *element);
+  NS_IMETHOD AddMessageDispositionState(nsIMsgDBHdr *aMessage, nsMsgDispositionState aDispositionFlag) MOZ_OVERRIDE;
+  NS_IMETHOD MarkMessagesRead(nsIArray *messages, bool markRead) MOZ_OVERRIDE;
+  NS_IMETHOD MarkAllMessagesRead(nsIMsgWindow *aMsgWindow) MOZ_OVERRIDE;
+  NS_IMETHOD MarkMessagesFlagged(nsIArray *messages, bool markFlagged) MOZ_OVERRIDE;
+  NS_IMETHOD MarkThreadRead(nsIMsgThread *thread) MOZ_OVERRIDE;
+  NS_IMETHOD SetLabelForMessages(nsIArray *aMessages, nsMsgLabelValue aLabel) MOZ_OVERRIDE;
+  NS_IMETHOD SetJunkScoreForMessages(nsIArray *aMessages, const nsACString& aJunkScore) MOZ_OVERRIDE;
+  NS_IMETHOD DeleteSubFolders(nsIArray *folders, nsIMsgWindow *msgWindow) MOZ_OVERRIDE;
+  NS_IMETHOD ReadFromFolderCacheElem(nsIMsgFolderCacheElement *element) MOZ_OVERRIDE;
+  NS_IMETHOD WriteToFolderCacheElem(nsIMsgFolderCacheElement *element) MOZ_OVERRIDE;
 
   NS_IMETHOD GetDBFolderInfoAndDB(nsIDBFolderInfo **folderInfo,
-                                  nsIMsgDatabase **db);
+                                  nsIMsgDatabase **db) MOZ_OVERRIDE;
   NS_IMETHOD DeleteMessages(nsIArray *messages,
                             nsIMsgWindow *msgWindow, bool
                             deleteStorage, bool isMove,
-                            nsIMsgCopyServiceListener* listener, bool allowUndo);
+                            nsIMsgCopyServiceListener* listener, bool allowUndo) MOZ_OVERRIDE;
   NS_IMETHOD CopyMessages(nsIMsgFolder *srcFolder,
                           nsIArray* messages,
                           bool isMove, nsIMsgWindow *msgWindow,
                           nsIMsgCopyServiceListener* listener, bool isFolder,
-                          bool allowUndo);
+                          bool allowUndo) MOZ_OVERRIDE;
   NS_IMETHOD CopyFolder(nsIMsgFolder *srcFolder, bool isMove, nsIMsgWindow *msgWindow,
-                        nsIMsgCopyServiceListener* listener);
+                        nsIMsgCopyServiceListener* listener) MOZ_OVERRIDE;
   NS_IMETHOD CopyFileMessage(nsIFile* file,
                               nsIMsgDBHdr* msgToReplace,
                               bool isDraftOrTemplate,
-                              PRUint32 aNewMsgFlags,
+                              uint32_t aNewMsgFlags,
                               const nsACString &aNewMsgKeywords,
                               nsIMsgWindow *msgWindow,
-                              nsIMsgCopyServiceListener* listener);
-  NS_IMETHOD GetNewMessages(nsIMsgWindow *aWindow, nsIUrlListener *aListener);
+                              nsIMsgCopyServiceListener* listener) MOZ_OVERRIDE;
+  NS_IMETHOD GetNewMessages(nsIMsgWindow *aWindow, nsIUrlListener *aListener) MOZ_OVERRIDE;
 
-  NS_IMETHOD GetFilePath(nsILocalFile** aPathName);
-  NS_IMETHOD SetFilePath(nsILocalFile * aPath);
+  NS_IMETHOD GetFilePath(nsIFile** aPathName) MOZ_OVERRIDE;
+  NS_IMETHOD SetFilePath(nsIFile * aPath) MOZ_OVERRIDE;
 
-  NS_IMETHOD Shutdown(bool shutdownChildren);
+  NS_IMETHOD Shutdown(bool shutdownChildren) MOZ_OVERRIDE;
 
-  NS_IMETHOD DownloadMessagesForOffline(nsIArray *messages, nsIMsgWindow *msgWindow);
+  NS_IMETHOD DownloadMessagesForOffline(nsIArray *messages, nsIMsgWindow *msgWindow) MOZ_OVERRIDE;
 
-  NS_IMETHOD DownloadAllForOffline(nsIUrlListener *listener, nsIMsgWindow *msgWindow);
-  NS_IMETHOD GetCanFileMessages(bool *aCanFileMessages);
-  NS_IMETHOD GetCanDeleteMessages(bool *aCanDeleteMessages);
-  NS_IMETHOD FetchMsgPreviewText(nsMsgKey *aKeysToFetch, PRUint32 aNumKeys,
+  NS_IMETHOD DownloadAllForOffline(nsIUrlListener *listener, nsIMsgWindow *msgWindow) MOZ_OVERRIDE;
+  NS_IMETHOD GetCanFileMessages(bool *aCanFileMessages) MOZ_OVERRIDE;
+  NS_IMETHOD GetCanDeleteMessages(bool *aCanDeleteMessages) MOZ_OVERRIDE;
+  NS_IMETHOD FetchMsgPreviewText(nsMsgKey *aKeysToFetch, uint32_t aNumKeys,
                                                  bool aLocalOnly, nsIUrlListener *aUrlListener,
-                                                 bool *aAsyncResults);
+                                                 bool *aAsyncResults) MOZ_OVERRIDE;
 
-  NS_IMETHOD AddKeywordsToMessages(nsIArray *aMessages, const nsACString& aKeywords);
-  NS_IMETHOD RemoveKeywordsFromMessages(nsIArray *aMessages, const nsACString& aKeywords);
+  NS_IMETHOD AddKeywordsToMessages(nsIArray *aMessages, const nsACString& aKeywords) MOZ_OVERRIDE;
+  NS_IMETHOD RemoveKeywordsFromMessages(nsIArray *aMessages, const nsACString& aKeywords) MOZ_OVERRIDE;
 
-  NS_IMETHOD NotifyCompactCompleted();
+  NS_IMETHOD NotifyCompactCompleted() MOZ_OVERRIDE;
+
+  // overrides nsMsgDBFolder::HasMsgOffline()
+  NS_IMETHOD HasMsgOffline(nsMsgKey msgKey, bool *_retval) MOZ_OVERRIDE;
+  // overrides nsMsgDBFolder::GetOfflineFileStream()
+  NS_IMETHOD GetOfflineFileStream(nsMsgKey msgKey, int64_t *offset, uint32_t *size, nsIInputStream **aFileStream) MOZ_OVERRIDE;
 
   NS_DECL_NSIMSGIMAPMAILFOLDER
   NS_DECL_NSIIMAPMAILFOLDERSINK
@@ -328,17 +300,27 @@ public:
   NS_DECL_NSIREQUESTOBSERVER
 
   // nsIUrlListener methods
-  NS_IMETHOD OnStartRunningUrl(nsIURI * aUrl);
-  NS_IMETHOD OnStopRunningUrl(nsIURI * aUrl, nsresult aExitCode);
+  NS_IMETHOD OnStartRunningUrl(nsIURI * aUrl) MOZ_OVERRIDE;
+  NS_IMETHOD OnStopRunningUrl(nsIURI * aUrl, nsresult aExitCode) MOZ_OVERRIDE;
 
   NS_DECL_NSIMSGFILTERHITNOTIFY
   NS_DECL_NSIJUNKMAILCLASSIFICATIONLISTENER
 
-  NS_IMETHOD IsCommandEnabled(const nsACString& command, bool *result);
-  NS_IMETHOD SetFilterList(nsIMsgFilterList *aMsgFilterList);
-  NS_IMETHOD GetCustomIdentity(nsIMsgIdentity **aIdentity);
+  NS_IMETHOD IsCommandEnabled(const nsACString& command, bool *result) MOZ_OVERRIDE;
+  NS_IMETHOD SetFilterList(nsIMsgFilterList *aMsgFilterList) MOZ_OVERRIDE;
+  NS_IMETHOD GetCustomIdentity(nsIMsgIdentity **aIdentity) MOZ_OVERRIDE;
 
-  nsresult AddSubfolderWithPath(nsAString& name, nsILocalFile *dbPath, nsIMsgFolder **child, bool brandNew = false);
+ /**
+  * This method is used to locate a folder where a msg could be present, not just
+  * the folder where the message first arrives, this method searches for the existence
+  * of msg in all the folders/labels that we retrieve from X-GM-LABELS also.
+  * overrides nsMsgDBFolder::GetOfflineMsgFolder()
+  *  @param msgKey key  of the msg for which we are trying to get the folder;
+  *  @param aMsgFolder  required folder;
+  */
+  NS_IMETHOD GetOfflineMsgFolder(nsMsgKey msgKey, nsIMsgFolder **aMsgFolder) MOZ_OVERRIDE;
+
+  nsresult AddSubfolderWithPath(nsAString& name, nsIFile *dbPath, nsIMsgFolder **child, bool brandNew = false);
   nsresult MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
                                   nsIMsgDatabase *sourceDB,
                                   const nsACString& destFolder,
@@ -348,39 +330,39 @@ public:
   // send notification to copy service listener.
   nsresult OnCopyCompleted(nsISupports *srcSupport, nsresult exitCode);
 
-  static nsresult  AllocateUidStringFromKeys(nsMsgKey *keys, PRUint32 numKeys, nsCString &msgIds);
+  static nsresult  AllocateUidStringFromKeys(nsMsgKey *keys, uint32_t numKeys, nsCString &msgIds);
   static nsresult  BuildIdsAndKeyArray(nsIArray* messages, nsCString& msgIds, nsTArray<nsMsgKey>& keyArray);
 
   // these might end up as an nsIImapMailFolder attribute.
-  nsresult SetSupportedUserFlags(PRUint32 userFlags);
-  nsresult GetSupportedUserFlags(PRUint32 *userFlags);
+  nsresult SetSupportedUserFlags(uint32_t userFlags);
+  nsresult GetSupportedUserFlags(uint32_t *userFlags);
 
   // Find the start of a range of msgKeys that can hold srcCount headers.
-  nsresult FindOpenRange(nsMsgKey &fakeBase, PRUint32 srcCount);
+  nsresult FindOpenRange(nsMsgKey &fakeBase, uint32_t srcCount);
 
 protected:
   // Helper methods
 
-  virtual nsresult CreateChildFromURI(const nsCString &uri, nsIMsgFolder **folder);
+  virtual nsresult CreateChildFromURI(const nsCString &uri, nsIMsgFolder **folder) MOZ_OVERRIDE;
   void FindKeysToAdd(const nsTArray<nsMsgKey> &existingKeys, nsTArray<nsMsgKey>
-    &keysToFetch, PRUint32 &numNewUnread, nsIImapFlagAndUidState *flagState);
+    &keysToFetch, uint32_t &numNewUnread, nsIImapFlagAndUidState *flagState);
   void FindKeysToDelete(const nsTArray<nsMsgKey> &existingKeys, nsTArray<nsMsgKey>
-    &keysToFetch, nsIImapFlagAndUidState *flagState, PRUint32 boxFlags);
+    &keysToFetch, nsIImapFlagAndUidState *flagState, uint32_t boxFlags);
   void PrepareToAddHeadersToMailDB(nsIImapProtocol* aProtocol);
   void TweakHeaderFlags(nsIImapProtocol* aProtocol, nsIMsgDBHdr *tweakMe);
 
   nsresult SyncFlags(nsIImapFlagAndUidState *flagState);
   nsresult HandleCustomFlags(nsMsgKey uidOfMessage, nsIMsgDBHdr *dbHdr,
-                             PRUint16 userFlags, nsCString& keywords);
+                             uint16_t userFlags, nsCString& keywords);
   nsresult NotifyMessageFlagsFromHdr(nsIMsgDBHdr *dbHdr, nsMsgKey msgKey,
-                                     PRUint32 flags);
+                                     uint32_t flags);
 
-  nsresult SetupHeaderParseStream(PRUint32 size, const nsACString& content_type, nsIMailboxSpec *boxSpec);
-  nsresult  ParseAdoptedHeaderLine(const char *messageLine, PRUint32 msgKey);
+  nsresult SetupHeaderParseStream(uint32_t size, const nsACString& content_type, nsIMailboxSpec *boxSpec);
+  nsresult  ParseAdoptedHeaderLine(const char *messageLine, uint32_t msgKey);
   nsresult  NormalEndHeaderParseStream(nsIImapProtocol *aProtocol, nsIImapUrl *imapUrl);
 
   void EndOfflineDownload();
-  nsresult CopyFileToOfflineStore(nsILocalFile *srcFile, nsMsgKey msgKey);
+  nsresult CopyFileToOfflineStore(nsIFile *srcFile, nsMsgKey msgKey);
 
   nsresult MarkMessagesImapDeleted(nsTArray<nsMsgKey> *keyArray, bool deleted, nsIMsgDatabase *db);
 
@@ -399,10 +381,10 @@ protected:
   nsresult DisplayStatusMsg(nsIImapUrl *aImapUrl, const nsAString& msg);
 
   //nsresult RenameLocal(const char *newName);
-  nsresult AddDirectorySeparator(nsILocalFile *path);
-  nsresult CreateSubFolders(nsILocalFile *path);
-  nsresult GetDatabase();
-  virtual void GetIncomingServerType(nsCString& serverType) { serverType.AssignLiteral("imap");}
+  nsresult AddDirectorySeparator(nsIFile *path);
+  nsresult CreateSubFolders(nsIFile *path);
+  nsresult GetDatabase() MOZ_OVERRIDE;
+  virtual void GetIncomingServerType(nsCString& serverType) MOZ_OVERRIDE { serverType.AssignLiteral("imap");}
 
   nsresult        GetFolderOwnerUserName(nsACString& userName);
   nsIMAPNamespace *GetNamespaceForFolder();
@@ -425,14 +407,14 @@ protected:
                           bool isMove,
                           bool selectedState,
                           bool acrossServers,
-                          PRUint32 newMsgFlags,
+                          uint32_t newMsgFlags,
                           const nsACString &newMsgKeywords,
                           nsIMsgCopyServiceListener* listener,
                           nsIMsgWindow *msgWindow,
                           bool allowUndo);
   nsresult GetMoveCoalescer();
   nsresult PlaybackCoalescedOperations();
-  virtual nsresult CreateBaseMessageURI(const nsACString& aURI);
+  virtual nsresult CreateBaseMessageURI(const nsACString& aURI) MOZ_OVERRIDE;
   // offline-ish methods
   nsresult GetClearedOriginalOp(nsIMsgOfflineImapOperation *op, nsIMsgOfflineImapOperation **originalOp, nsIMsgDatabase **originalDB);
   nsresult GetOriginalOp(nsIMsgOfflineImapOperation *op, nsIMsgOfflineImapOperation **originalOp, nsIMsgDatabase **originalDB);
@@ -472,18 +454,18 @@ protected:
   /// the junk destination folder
   nsCOMPtr<nsIMsgFolder> mSpamFolder;
   nsMsgKey m_curMsgUid;
-  PRUint32 m_uidValidity;
+  uint32_t m_uidValidity;
 
   // These three vars are used to store counts from STATUS or SELECT command
   // They include deleted messages, so they can differ from the generic
   // folder total and unread counts.
-  PRInt32 m_numServerRecentMessages;
-  PRInt32 m_numServerUnseenMessages;
-  PRInt32 m_numServerTotalMessages;
+  int32_t m_numServerRecentMessages;
+  int32_t m_numServerUnseenMessages;
+  int32_t m_numServerTotalMessages;
   // if server supports UIDNEXT, we store it here.
-  PRInt32 m_nextUID;
+  int32_t m_nextUID;
 
-  PRInt32  m_nextMessageByteLength;
+  int32_t  m_nextMessageByteLength;
   nsCOMPtr<nsIUrlListener> m_urlListener;
   bool m_urlRunning;
 
@@ -491,7 +473,7 @@ protected:
   nsRefPtr<nsMsgTxn> m_pendingUndoTxn;
   nsRefPtr<nsImapMailCopyState> m_copyState;
   char m_hierarchyDelimiter;
-  PRInt32 m_boxFlags;
+  int32_t m_boxFlags;
   nsCString m_onlineFolderName;
   nsCString m_ownerUserName;  // username of the "other user," as in
   // "Other Users' Mailboxes"
@@ -514,9 +496,11 @@ protected:
   bool m_expunging;
   bool m_applyIncomingFilters; // apply filters to this folder, even if not the inbox
   nsMsgIMAPFolderACL *m_folderACL;
-  PRUint32     m_aclFlags;
-  PRUint32     m_supportedUserFlags;
+  uint32_t     m_aclFlags;
+  uint32_t     m_supportedUserFlags;
 
+  // determines if we are on GMail server
+  bool m_isGmailServer;
   // offline imap support
   bool m_downloadingFolderForOfflineUse;
   bool m_filterListRequiresBody;
@@ -526,8 +510,8 @@ protected:
 
   // Quota support
   nsCString m_folderQuotaRoot;
-  PRUint32 m_folderQuotaUsedKB;
-  PRUint32 m_folderQuotaMaxKB;
+  uint32_t m_folderQuotaUsedKB;
+  uint32_t m_folderQuotaMaxKB;
 
   // Pseudo-Offline Playback support
   nsPlaybackRequest *m_pendingPlaybackReq;
@@ -538,7 +522,7 @@ protected:
   nsDataHashtable<nsCStringHashKey, nsMsgKey> m_pseudoHdrs;
 
   nsTArray<nsMsgKey> m_keysToFetch;
-  PRUint32 m_totalKeysToFetch;
+  uint32_t m_totalKeysToFetch;
 
 };
 #endif
