@@ -51,7 +51,6 @@
 #include "nsIMsgCopyService.h"
 #include "nsICryptoHash.h"
 #include "nsIStringBundle.h"
-#include "nsLocalStrings.h"
 #include "nsIMsgFilterPlugin.h"
 #include "nsIMutableArray.h"
 #include "nsArrayUtils.h"
@@ -118,8 +117,7 @@ NS_IMETHODIMP nsMsgMailboxParser::OnStartRequest(nsIRequest *request, nsISupport
           path->GetFileSize(&fileSize);
             // the size of the mailbox file is our total base line for measuring progress
             m_graph_progress_total = (uint32_t) fileSize;
-            UpdateStatusText(LOCAL_STATUS_SELECTING_MAILBOX);
-
+            UpdateStatusText("buildingSummary");
             nsCOMPtr<nsIMsgDBService> msgDBService = do_GetService(NS_MSGDB_SERVICE_CONTRACTID, &rv);
             if (msgDBService)
             {
@@ -176,7 +174,7 @@ NS_IMETHODIMP nsMsgMailboxParser::OnStopRequest(nsIRequest *request, nsISupports
     // be sure to clear any status text and progress info..
     m_graph_progress_received = 0;
     UpdateProgressPercent();
-    UpdateStatusText(LOCAL_STATUS_DOCUMENT_DONE);
+    UpdateStatusText("localStatusDocumentDone");
 
     return NS_OK;
 }
@@ -279,7 +277,7 @@ nsresult nsMsgMailboxParser::Init()
   return AcquireFolderLock();
 }
 
-void nsMsgMailboxParser::UpdateStatusText (uint32_t stringID)
+void nsMsgMailboxParser::UpdateStatusText (const char* stringName)
 {
   if (m_statusFeedback)
   {
@@ -293,14 +291,9 @@ void nsMsgMailboxParser::UpdateStatusText (uint32_t stringID)
     if (NS_FAILED(rv))
       return;
     nsString finalString;
-    if (stringID == LOCAL_STATUS_SELECTING_MAILBOX)
-    {
-      const PRUnichar * stringArray[] = { m_folderName.get() };
-      rv = bundle->FormatStringFromID(stringID, stringArray, 1,
-                                      getter_Copies(finalString));
-    }
-    else
-      bundle->GetStringFromID(stringID, getter_Copies(finalString));
+    const PRUnichar * stringArray[] = { m_folderName.get() };
+    rv = bundle->FormatStringFromName(NS_ConvertASCIItoUTF16(stringName).get(),
+                                      stringArray, 1, getter_Copies(finalString));
     m_statusFeedback->ShowStatusString(finalString);
   }
 }
@@ -438,7 +431,7 @@ void nsMsgMailboxParser::OnNewMessage(nsIMsgWindow *msgWindow)
   Clear();
 }
 
-nsresult nsMsgMailboxParser::HandleLine(char *line, uint32_t lineLength)
+nsresult nsMsgMailboxParser::HandleLine(const char *line, uint32_t lineLength)
 {
   /* If this is the very first line of a non-empty folder, make sure it's an envelope */
   if (m_graph_progress_received == 0)
@@ -881,12 +874,11 @@ void nsParseMailMessageState::GetAggregateHeader (nsVoidArray &list, struct mess
   {
     header = (struct message_header*) list.ElementAt(i);
     length += (header->length + 1); //+ for ","
-    NS_ASSERTION(header->length == (int32_t)strlen(header->value), "header corrupted");
   }
 
   if (length > 0)
   {
-    char *value = (char*) PR_MALLOC (length + 1); //+1 for null term
+    char *value = (char*) PR_CALLOC (length + 1); //+1 for null term
     if (value)
     {
       // Catenate all the To lines together, separated by commas
@@ -895,7 +887,7 @@ void nsParseMailMessageState::GetAggregateHeader (nsVoidArray &list, struct mess
       for (i = 0; i < size; i++)
       {
         header = (struct message_header*) list.ElementAt(i);
-        PL_strcat (value, header->value);
+        PL_strncat (value, header->value, header->length);
         if (i + 1 < size)
           PL_strcat (value, ",");
       }
@@ -1053,7 +1045,7 @@ nsresult nsParseMailMessageState::ParseHeaders ()
 
       ToLowerCase(headerStr);
       uint32_t customHeaderIndex = m_customDBHeaders.IndexOf(headerStr);
-      if (customHeaderIndex != nsTArray<nsCString>::NoIndex)
+      if (customHeaderIndex != m_customDBHeaders.NoIndex)
         header = & m_customDBHeaderValues[customHeaderIndex];
     }
 
@@ -1063,7 +1055,7 @@ nsresult nsParseMailMessageState::ParseHeaders ()
 SEARCH_NEWLINE:
     // move past any non terminating characters, rewriting them if folding white space
     // exists
-    while (buf < buf_end && *buf != '\r' && *buf != '\n')
+    while (buf <= buf_end && *buf != '\r' && *buf != '\n')
     {
       if (writeOffset)
         *(buf - writeOffset) = *buf;
@@ -1108,6 +1100,8 @@ SEARCH_NEWLINE:
 
       header->value = value;
       header->length = buf - header->value - writeOffset;
+      if (header->length < 0)
+        header->length = 0;
     }
     if (*buf == '\r' || *buf == '\n')
     {
@@ -1152,8 +1146,7 @@ SEARCH_NEWLINE:
         // Someone might want the received header saved.
         if (m_customDBHeaders.Length())
         {
-          uint32_t customHeaderIndex = m_customDBHeaders.IndexOf(NS_LITERAL_CSTRING("received"));
-          if (customHeaderIndex != nsTArray<nsCString>::NoIndex)
+          if (m_customDBHeaders.Contains(NS_LITERAL_CSTRING("received")))
           {
             if (!m_receivedValue.IsEmpty())
               m_receivedValue.Append(' ');
@@ -1223,8 +1216,6 @@ nsresult nsParseMailMessageState::InternSubject (struct message_header *header)
     return NS_OK;
   }
 
-  NS_ASSERTION (header->length == (short) strlen(header->value), "subject corrupt while parsing message");
-
   key = (char *) header->value;  /* #### const evilness */
 
   L = header->length;
@@ -1312,7 +1303,7 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
   recipient  = (to.length         ? &to :
   cc.length         ? &cc :
   m_newsgroups.length ? &m_newsgroups :
-  sender);
+  0);
   ccList     = (cc.length ? &cc : 0);
   bccList    = (m_bccList.length    ? &m_bccList    : 0);
   subject    = (m_subject.length    ? &m_subject    : 0);
@@ -1449,7 +1440,6 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
       group in the summary list, and only being able to sort on the first
         group rather than the whole list.  It's worth it. */
         char * ch;
-        NS_ASSERTION (recipient->length == (uint16_t) strlen(recipient->value), "invalid recipient");
         ch = PL_strchr(recipient->value, ',');
         if (ch)
         {
@@ -2595,7 +2585,7 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
 
   if (movedMsgIsNew)
     destIFolder->SetHasNewMessages(true);
-  if (m_filterTargetFolders.IndexOf(destIFolder) == -1) // IndexOf() type?
+  if (!m_filterTargetFolders.Contains(destIFolder))
     m_filterTargetFolders.AppendObject(destIFolder);
 
   destIFolder->ReleaseSemaphore (myISupports);
